/*
 * A simple Lexer for SML/NG
 */

#include "smlng.h"
#include <iostream>
#include <iterator>

using std::cin;
using std::istreambuf_iterator;

Ws curws;
Text curtxt;
BoldTag curbtag;
StrgTag curstag;
PlainTag curptag;
ColorTag curctag;
SizeTag cursztag;
UlineTag curutag;
EmphTag curetag;
ItalTag curitag;
TtypeTag curttag;

istreambuf_iterator<char> inpos(cin);
istreambuf_iterator<char> endpos;

unsigned long numinchars;

Token*
next_token()
{
  char readchar;
  Token* retval;

  if (inpos != endpos) {
    numinchars ++;
    switch (readchar = *inpos++) {
    case '<': // It's a tag - find out what tag it really is
      {
        bool isclosetag = false;
        char tagchar;

        numinchars ++;

        // Check if it's a closing tag
        if ( (tagchar = *inpos++) == '/' ) {
          numinchars ++;
          isclosetag = true;
          tagchar = *inpos++;
        }
        
        switch (tagchar) {
        case 'B':
          {
            curbtag.closetag = isclosetag;
            retval = &curbtag;
            break;
          }
        case 'S':
          {
            curstag.closetag = isclosetag;
            retval = &curstag;
            break;
          }
        case 'I':
          {
            curitag.closetag = isclosetag;
            retval = &curitag;
            break;
          }
        case 'U':
          {
            curutag.closetag = isclosetag;
            retval = &curutag;
            break;
          }
        case 'E':
          {
            curetag.closetag = isclosetag;
            numinchars++;
            inpos++;
            retval = &curetag;
            break;
          }
        case 'P':
          {
            curptag.closetag = isclosetag;
            numinchars++;
            inpos++;
            retval = &curptag;
            break;
          }
        case 'T':
          {
            curttag.closetag = isclosetag;
            numinchars++;
            inpos++;
            retval = &curttag;
            break;
          }
        case 'r':
          {
            curctag.closetag = isclosetag;
            curctag.color = red;
            retval = &curctag;
            break;
          }
        case 'g':
          {
            curctag.closetag = isclosetag;
            curctag.color = green;
            retval = &curctag;
            break;
          }
        case 'b':
          {
            curctag.closetag = isclosetag;
            curctag.color = blue;
            retval = &curctag;
            break;
          }
        case 'c':
          {
            curctag.closetag = isclosetag;
            curctag.color = cyan;
            retval = &curctag;
            break;
          }
        case 'm':
          {
            curctag.closetag = isclosetag;
            curctag.color = magenta;
            retval = &curctag;
            break;
          }
        case 'y':
          {
            curctag.closetag = isclosetag;
            curctag.color = yellow;
            retval = &curctag;
            break;
          }
        case 'k':
          {
            curctag.closetag = isclosetag;
            curctag.color = black;
            retval = &curctag;
            break;
          }
        case 'w':
          {
            curctag.closetag = isclosetag;
            curctag.color = white;
            retval = &curctag;
            break;
          }
        default: /* Which actually is 0..9 */
          {
            cursztag.closetag = isclosetag;
            cursztag.size = tagchar - '0';
            retval = &cursztag;
          }
        } /* End of nested switch */

        // Skip the closing '>'
        numinchars++;
        inpos++;
        break;
      } /* End of case '<' */
    case SPC: case CR: case LF: case TAB: // It's apparently a white space
      {
        curws.wschar = readchar;
        retval = &curws;
        break;
      }
    default: // If neither of the above - it's a text char
      {
        curtxt.txtchar = readchar;
        retval = &curtxt;
      }
    } /* End of Outer switch statement */
  } /* End of if (inpos != endpos) */
  else { retval = 0; }

  return retval;
}
