/* -*-c++-*- */

/*
 * Declarations for SML/NG parser and lexer
 *
 */

#include <ext/rope>
#include <list>
#include <string>

using std::crope;
using std::list;
using std::string;

struct Token {
  virtual void handle() = 0;
}; /* End of struct Token */

struct Ws : public Token {
  void handle();
  char wschar;  
}; /* End of struct Ws */

struct Text : public Token {
  void handle();
  char txtchar;  
}; /* End of struct Text */

struct BoldTag : public Token {
  void handle();
  bool closetag;
}; /* End of struct BoldTag */

struct StrgTag : public Token {
  void handle();
  bool closetag;
}; /* End of struct StrgTag */

struct EmphTag : public Token {
  void handle();
  bool closetag;
}; /* End of struct EmphTag */

struct ItalTag : public Token {
  void handle();
  bool closetag;
}; /* End of struct ItalTag */

struct PlainTag : public Token {
  void handle();
  bool closetag;
}; /* End of struct PlainTag */

struct TtypeTag : public Token {
  void handle();
  bool closetag;
}; /* End of struct TtypeTag */

struct UlineTag : public Token {
  void handle();
  bool closetag;
}; /* End of struct UlineTag */

struct SizeTag : public Token {
  void handle();
  bool closetag;
  unsigned char size;
}; /* End of struct SizeTag */

struct ColorTag : public Token {
  void handle();
  bool closetag;
  unsigned char color;
}; /* End of struct ColorTag */

Token* next_token();

// Data declarations

const char SPC = ' ';
const char TAB = '\t';
const char CR = '\r';
const char LF = '\n';

const char COLORS[] = {'r', 'g', 'b', 'c', 'm', 'y', 'k', 'w' };

const unsigned char red = 0;
const unsigned char green = 1;
const unsigned char blue = 2;
const unsigned char cyan = 3;
const unsigned char magenta = 4;
const unsigned char yellow = 5;
const unsigned char black = 6;
const unsigned char white = 7;

extern Ws curws;
extern Text curtxt;
extern BoldTag curbtag;
extern StrgTag curstag;
extern PlainTag curptag;
extern ColorTag curctag;
extern SizeTag cursztag;
extern UlineTag curutag;
extern EmphTag curetag;
extern ItalTag curitag;
extern TtypeTag curttag;

extern unsigned long numinchars;
