/*
 * All tag handlers except for color and size
 *
 */

#include "handlers.h"

void BoldTag::handle() {
  static unsigned long ignoreclosetag = 0; // To eliminate redundant nested tags
  
  if (!closetag) {
    // It's a open bold tag
    if (!nextprec.isbold) {
      // okay, the present context is not bold
      nextprec.isbold = true;
      //outstr += "<B>";
      if (closetagsInTaglist('B')) curtaglist.pop_back();
      else curtaglist.push_back("<B>");
    } else ignoreclosetag++; // Bold is already set - don't set it again for next context
  } else {
    // It's a closed bold tag
    if (ignoreclosetag) ignoreclosetag --;
    else {
      // We did not ignore the corresponding open tag - so reflect changes in next context
      nextprec.isbold = false;
      //outstr += "</B>";
      if (opentagsInTaglist()) curtaglist.pop_back();
      else curtaglist.push_back("</B>");
    }
  }
}

void StrgTag::handle() {
  static unsigned long ignoreclosetag = 0; // To eliminate redundant nested tags
  
  if (!closetag) {
    // It's a open strong tag
    if (!nextprec.isstrg) {
      // okay, the present context is not strong
      nextprec.isstrg = true;
      //outstr += "<S>";
      if (closetagsInTaglist('S')) curtaglist.pop_back();
      else curtaglist.push_back("<S>");
    } else ignoreclosetag++; // Strong is already set - don't set it again for next context
  } else {
    // It's a closed strong tag
    if (ignoreclosetag) ignoreclosetag --;
    else {
      // We did not ignore the corresponding open tag - so reflect changes in next context
      nextprec.isstrg = false;
      //outstr += "</S>";
      if (opentagsInTaglist()) curtaglist.pop_back();
      else curtaglist.push_back("</S>");
    }
  }
}

void EmphTag::handle() {
  static unsigned long ignoreclosetag = 0; // To eliminate emphasis tags within Strong

  if (!closetag) {
    // It's a open emphasis tag
    if (!nextprec.isstrg) {
      // Emphasis is valid only if the strong is not set in the present context
      // And in that case, it is always valid because it flip-flops unlike other flags
      nextprec.isemph = !nextprec.isemph;
      //outstr += "<EM>";
      if (closetagsInTaglist('E')) {
        curtaglist.pop_back();
      } else {
        if (skipnextem) {
          curtaglist.remove("<EM>");
          skipnextem = false;
          ignoreclosetag += 2;
        } else {
          curtaglist.push_back("<EM>");
          skipnextem = true;
        }
      }
    } else ignoreclosetag ++;
  } else {
    // It's a closed emphasis tag
    if (ignoreclosetag) ignoreclosetag --;
    else {
      nextprec.isemph = !nextprec.isemph;
      //outstr += "</EM>";
      if (opentagsInTaglist()) curtaglist.pop_back();
      else curtaglist.push_back("</EM>");
    }
  }
}

void ItalTag::handle() {
  static unsigned long ignoreclosetag = 0; // To eliminate redundant nested tags

  if (!closetag) {
    // It's a open italics tag
    if (!nextprec.isital) {
      // okay, the present context is not italics
      nextprec.isital = true;
      //outstr += "<I>";
      if (closetagsInTaglist('I')) curtaglist.pop_back();
      else curtaglist.push_back("<I>");
    } else ignoreclosetag++; // italics is already set - don't set it again for next context
  } else {
    // It's a closed italics tag
    if (ignoreclosetag) ignoreclosetag --;
    else {
      // We did not ignore the corresponding open tag - so reflect changes in next context
      nextprec.isital = false;
      //outstr += "</I>";
      if (opentagsInTaglist()) curtaglist.pop_back();
      else curtaglist.push_back("</I>");
    }
  }
}

void UlineTag::handle() {
  static unsigned long ignoreclosetag = 0; // To eliminate underline nestings > 3

  if (!closetag) {
    // It's a open underline tag
    
    if (nextprec.ulinelevel < 4) {
      // okay, underline level needs to be incremented
      (nextprec.ulinelevel) ++;
      //outstr += "<U>";
      if (closetagsInTaglist('U')) curtaglist.pop_back();
      else curtaglist.push_back("<U>");
    } else ignoreclosetag++; // Bold is already set - don't set it again for next context
  } else {
    // It's a closed bold tag
    if (ignoreclosetag) ignoreclosetag --;
    else {
      // We did not ignore the corresponding open tag - so reflect changes in next context
      (nextprec.ulinelevel) --;
      //outstr += "</U>";
      if (opentagsInTaglist()) curtaglist.pop_back();
      else curtaglist.push_back("</U>");
    }
  }
}

void TtypeTag::handle() {
  static unsigned long ignoreclosetag = 0; // To eliminate redundant nested tags
  
  if (!closetag) {
    if (!nextprec.isttype) {
      // Set Typewriter only if it is not already set
      nextprec.isttype = true;
      //outstr += "<TT>";
      if (closetagsInTaglist('T')) curtaglist.pop_back();
      else curtaglist.push_back("<TT>");
    } else ignoreclosetag++; // if redundant remember to ignore the corresponding close tag
  } else {
    if (ignoreclosetag) ignoreclosetag--;
    else {
      nextprec.isttype = false;
      //outstr += "</TT>";
      if (opentagsInTaglist()) curtaglist.pop_back();
      else curtaglist.push_back("</TT>");
    }
  }
}

void PlainTag::handle() {
  static unsigned long ignoreclosetag = 0; // To eliminate redundant nested tags
  static Prec oldprec;
  
  if (!closetag) {
    if (!nextprec.isPlain()) {
      // Set Plain only if it is not already set
      oldprec = nextprec; // We need this to be able to restore context after </PL>
      nextprec.isbold = false;
      nextprec.isital = false;
      nextprec.isstrg = false;
      nextprec.isemph = false;
      nextprec.isttype = false;
      nextprec.ulinelevel = 0;
      //outstr += "<PL>";
      if (closetagsInTaglist('P')) curtaglist.pop_back();
      else curtaglist.push_back("<PL>");
    } else ignoreclosetag++; // if redundant remember to ignore the corresponding close tag
  } else {
    if (ignoreclosetag) ignoreclosetag--;
    else {
      nextprec = oldprec;
      //outstr += "</PL>";
      if (opentagsInTaglist()) curtaglist.pop_back();
      else curtaglist.push_back("</PL>");
    }
  }
}
